@@ -25,9 +25,10 @@ class MinimalHttpClient:
 
 
 class MinimalResponse(object):  # Not for production use
-    def __init__(self, requests_resp=None, status_code=None, text=None):
+    def __init__(self, requests_resp=None, status_code=None, text=None, headers=None):
         self.status_code = status_code or requests_resp.status_code
-        self.text = text or requests_resp.text
+        self.text = text if text is not None else requests_resp.text
+        self.headers = {} if headers is None else headers
         self._raw_resp = requests_resp
 
     def raise_for_status(self):
